
import ice.storm.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;
import org.w3c.dom.events.*;

// HTML4 pilot, doc, DOM impl
import ice.pilots.html4.*;

import java.net.URL;

import java.util.Hashtable;
import java.io.StringReader;


/**
* This is a simple Web Browser application. It is slightly more advanced
* than the simple viewer located in the examples subdirectory. This
* browser utilizes StormCallback to provide support for new top-level
* windows creation.
* This example also demonstrates on how to register DOM event listeners
* for link clicks
*/

public class TestIceBrowser {

	public static void main(String args[]) {

	new MyBrowser();
       	}

}


class MyBrowser {

	private StormBase base;

	public MyBrowser() {

		base = new StormBase();
		base.setCallback(new MyCallback());


		String style = ":link {text-decoration: none;}";

		DStyleSheet us = ThePilot.getUserStyleSheet();
		CSSBuilder b = new CSSBuilder();
		try {
			b.build(us,new StringReader(style),null);
		}	
		catch(Exception ex) {}

		String loc = "http://www.gatewayportal.org";
		//if (args.length>0) loc=args[0];

		base.renderContent(loc,null,"demo");
		//base.addPropertyChangeListener(this,"demo");
	}


	//public void propertyChange(PropertyChangeEvent e) {
	  //System.out.println("Event invoked .....");
	
	//}


}


class MyCallback implements StormCallback {

	private Hashtable windows = new Hashtable();
	private StormBase base;

	public void init(StormBase base) {
		this.base=base;
	}

	public Container createTopLevelContainer(Viewport viewport) {
		BrowserFrame f = new BrowserFrame(base,viewport);
		windows.put(viewport.getId(),f);
		f.setVisible(true);
		return f.getPanel();
	}

	public void disposeTopLevelContainer(Viewport viewport) {
		BrowserFrame f = (BrowserFrame)windows.get(viewport.getId());
		if (f!=null) {
			windows.remove(viewport.getId());
			f.setVisible(false);
			f.dispose();
		}

		if (windows.isEmpty()) {
			System.exit(0);
		}
	}

	public void message(Viewport vp, String msg) {
		Frame f = findFrame(vp);
		if (f == null) return;

		MessageBox b = new MessageBox(f,msg,true);
		packAndShow(b,f);
	}

	public boolean confirm(Viewport vp, String msg) {
		Frame f = findFrame(vp);
		if (f == null) return false;

		ConfirmBox b = new ConfirmBox(f,msg,true);
		packAndShow(b,f);

		return b.response;
	}

	public String prompt(Viewport vp, String msg, String defVal) {

		Frame f = findFrame(vp);
		if (f == null) return null;

		PromptBox b = new PromptBox(f,msg,true,defVal);
		packAndShow(b,f);

		return b.responseString;
	}

	private Frame findFrame(Viewport vp) {
		Component x = (Component)vp.getContainer();
		if (x == null) return null;

		while(x.getParent() != null && !(x instanceof Frame)) {
			x = x.getParent();
		}
		if (!(x instanceof Frame)) return null;

		return (Frame)x;
	}

	private void packAndShow(Dialog d, Frame f) {
		d.pack();
		if (f!=null) {
			Point p = f.getLocation();
			Dimension fd = f.getSize();
			Dimension dd = d.getSize();
			int x = p.x+(fd.width-dd.width)/2;
			if (x<0) x=0;
			int y = p.y+(fd.height-dd.height)/2;
			if (y<0) y=0;
			d.setLocation(x,y);
		}
		d.show();
		d.toFront();
	}




}



class BrowserFrame extends Frame implements
			ActionListener, 
			WindowListener, 
			PropertyChangeListener {


	private StormBase base;
	private String viewportId;
	private Panel panel;

	private TextField loc;
	private Button back, forward;
	private Label status;


	public BrowserFrame(StormBase b, Viewport viewport) {
		super("Storm - "+viewport.getName());
		this.base=b;
		this.viewportId=viewport.getId();
		setLayout(new BorderLayout());

		addWindowListener(this);

		base.addPropertyChangeListener(this,viewportId);

		Font f = new Font("SansSerif",Font.PLAIN,12);

		back = new Button("back");
		back.setFont(f);
		back.setBackground(Color.lightGray);
		back.addActionListener(this);
		forward = new Button("forward");
		forward.setFont(f);
		forward.setBackground(Color.lightGray);
		forward.addActionListener(this);


		loc = new TextField("",30);
		loc.setFont(f);
		loc.setBackground(Color.white);
		loc.setForeground(Color.black);
		loc.addActionListener(this);

		status = new Label("SimpleBrowser demo",Label.CENTER);
		status.setFont(f);
		status.setBackground(Color.gray);
		status.setForeground(Color.black);


	// Buttons in a panel
		Panel p = new Panel();
		p.setLayout(new FlowLayout(FlowLayout.LEFT,0,0));
		p.add(back);
		p.add(forward);

	// Panel with buttons and location
		Panel p2 = new Panel();
		p2.setBackground(Color.black);
		p2.setLayout(new BorderLayout(0,0));
		p2.add(p,"Center");
		p2.add(loc,"South");

		add(p2,"North");

		panel = new Panel();
		panel.setLayout(new BorderLayout(0,0));
		add(panel,"Center");

		add(status,"South");

		setSize(800,650);
	}

	Panel getPanel() {return panel;}


	public void actionPerformed(ActionEvent e) {

		Object src=e.getSource();

		if (src==loc) {
			String vname=viewportId;
			String str = loc.getText();

			base.renderContent(str,null,vname);
		}
		else if (src==back) {
			base.goBack(viewportId);
		}
		else if (src==forward) {
			base.goForward(viewportId);
		}
	}

// PropertyChangeListener implementation

	public void propertyChange(PropertyChangeEvent e) {

		System.out.println("PropertyChange invoked ...");

		Viewport v = (Viewport)e.getSource();
		boolean isMyView=v.getId().equals(viewportId);

		String name = e.getPropertyName();
		System.out.println("Name: "+name);

		if (isMyView&&name.equals("title")) {
			setTitle("ICE Storm - "+v.getName()+" - "+e.getNewValue());
		}
		else if (isMyView&&name.equals("location")) {
			loc.setText((String)e.getNewValue());
			System.out.println("Location: "+(String)e.getNewValue());
		}
		else if (name.equals("statusLine")) {
			status.setText((String)e.getNewValue());
		}
		else if (name.equals("contentLoadingProgress")) {
			try {
				String val = (String)e.getNewValue();
				int idx1=val.indexOf(' ');
				int idx2=val.indexOf(' ',idx1+1);

				int l = Integer.parseInt(val.substring(idx1+1,idx2));
				int t = Integer.parseInt(val.substring(idx2+1));

				status.setText("loading "+l+" "+t);
//				status.updateProgress(val.substring(0,idx1),l,t);
			}
			catch(Exception ex) {
				//ex.printStackTrace();
				System.out.println("PropertyChange....CallBack");
			}

		}


/*		else if (name.equals("contentLoading")) {
			String str = (String)e.getNewValue();
			if (str.equals("end")) {
System.out.println("registering dom listeners for links");
				Pilot p = v.getPilot();
				if (p instanceof ice.pilots.html4.ThePilot) {
					ice.pilots.html4.ThePilot pp = (ice.pilots.html4.ThePilot)p;

					HTMLCollection l = ((ice.pilots.html4.DDocument)pp.getDocument()).getLinks();
					for(int i=0;i<l.getLength();i++) {
						Node n = l.item(i);
						((EventTarget)n).addEventListener("click",new listen_link(),false);
					}

					l = ((ice.pilots.html4.DDocument)pp.getDocument()).getForms();
					for(int i=0;i<l.getLength();i++) {
						Node n = l.item(i);
						((EventTarget)n).addEventListener("submit",new listen_form(),false);
					}

				}
			}
		}
*/
		else if(name.equals("outstandingImages")) {

			String val1 = (String)e.getNewValue();
			if (val1.equals("0"))
				status.setText("loading images: done");
			else
				status.setText("loading images: "+val1+" left");

		}
		else { 
			status.setText(name+": "+(String)e.getNewValue());
		}
	}


// WindowListener implementation (remove this???)
        public void windowClosing(WindowEvent e) {
		base.closeViewport(viewportId);
        }

	public void windowClosed(WindowEvent ev) {}
 	public void windowOpened(WindowEvent ev) {}
	public void windowIconified(WindowEvent ev) {}
	public void windowDeiconified(WindowEvent ev) {}
	public void windowActivated(WindowEvent ev) {}
	public void windowDeactivated(WindowEvent ev) {}
}



class listen_link implements org.w3c.dom.events.EventListener {

	public void handleEvent(org.w3c.dom.events.Event e) {
		System.out.println(" event - "+e.getType()+" target="+e.getTarget()+" curNode="+e.getCurrentNode());

		DAnchorElement ae = (DAnchorElement)e.getTarget();
		DDocument doc = (DDocument)ae.getOwnerDocument();
		URL fullURL = doc.resolveURL(ae.getHref());
System.out.println("     full url = "+fullURL);
	}
}


class listen_form implements org.w3c.dom.events.EventListener {

	public void handleEvent(org.w3c.dom.events.Event e) {
		System.out.println(" event - "+e.getType()+" target="+e.getTarget()+" curNode="+e.getCurrentNode());

		DFormElement fe = (DFormElement)e.getTarget();
System.out.println("     data for submit = "+fe.getSubmitData());

	}
}



class MessageBox extends Dialog implements ActionListener {

    public MessageBox(Frame topLevel, String titleIn, boolean modal) {
        super(topLevel,"ICE Browser",modal);

        Panel p = new Panel();
        p.add(new Label(titleIn));
        add(p,"North");
        Button b = new Button("OK");
        b.addActionListener(this);
        add(b,"South");
    }
    public void actionPerformed(ActionEvent e) {
	setVisible(false); dispose();
    }
}

class ConfirmBox extends Dialog implements ActionListener {

    private Button okButton=new Button("OK");
    private Button cancelButton=new Button("Cancel");
    boolean response=false;


    public ConfirmBox(Frame topLevel, String messageIn, boolean modal) {
        super(topLevel,"ICE Browser",modal);

        Panel p = new Panel();
        p.add(okButton);
        p.add(cancelButton);

        okButton.addActionListener(this);
        cancelButton.addActionListener(this);

        add(p,"South");
        add(new Label(messageIn),"North");
    }

    public void actionPerformed(ActionEvent e) {
        response=(e.getSource()==okButton);
        setVisible(false);
        dispose();
    }
}

class PromptBox extends Dialog implements ActionListener {

    private Button okButton=new Button("OK");
    private Button cancelButton=new Button("Cancel");
    private TextField theText=null;
    String responseString=null;

    public PromptBox(Frame topLevel, String messageIn, boolean modal,String defaultValueIn) {
        super(topLevel,"ICE Browser",modal);

        theText=new TextField(defaultValueIn,30);
        theText.addActionListener(this);

        Panel p = new Panel();
        p.setLayout(new GridLayout(2,1));
        p.add(new Label(messageIn));
        p.add(theText);
        add(p,"North");

        okButton.addActionListener(this);
        cancelButton.addActionListener(this);

        p = new Panel();
        p.add(okButton);
        p.add(cancelButton);
        add(p,"South");
    }

    public void actionPerformed(ActionEvent e) {
        Object src = e.getSource();
        if (src==okButton||src==theText) {
            responseString=theText.getText();
        }
        setVisible(false);
        dispose();
    }
}
